//Copyright (C) 2004, Brian Enigma <enigma at netninja.com>
//This file is part of MagicCodes.
//
//MagicCodes is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.
//
//MagicCodes is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with Foobar; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package org.ninjasoft.magiccodes.logic;

/**
 * Perform various translations
 * 
 * @author enigma
 */
public class Translator {
    /**
     * Convert a byte array to a binary string
     * @param c
     * @return
     */
    public static String byteToBin(int[] in) {
        StringBuffer result = new StringBuffer(in.length * 8);
        for (int pos=0; pos < in.length; pos++) {
            StringBuffer byteResult = new StringBuffer(8);
            int mask = 1;
            for (int i=0; i<8; i++) {
                if ((in[pos] & mask) == mask)
                    byteResult.insert(0, "1");
                else
                    byteResult.insert(0, "0");
                mask = mask << 1;
            }
            result.append(byteResult);
        }
        return result.toString();
    }

    /**
     * Convert a string of binary digits (0's and 1's) into a byte array
     * @param bin
     * @return
     * @throws FormatException
     */
    public static int[] binToByte(String bin) throws FormatException {
        bin = normalizeString(bin);
        int len = bin.length() / 8;
        int[] result = new int[len];
        if (bin.length() % 8 != 0)
            throw new FormatException("Length of binary data is not evenly divisible by 8!");
        for (int charPos=0; charPos < len; charPos++) {
            int value = 0;
            for (int bitPos = 0; bitPos < 8; bitPos++) {
                char c = bin.charAt((charPos * 8) + bitPos);
                //if (bitPos > 0)
                	value *= 2;
                if (c == '0')
                    value += 0;
                else if (c == '1')
                    value += 1;
                else
                    throw new FormatException("Non-binary character found in data: " + c);
            }
            result[charPos] = value;
        }
        return result;
    }
    
    /**
     * Convert a byte array to a hex string
     * @param in
     * @return
     */
    public static String byteToHex(int[] in) {
        StringBuffer result = new StringBuffer(in.length * 2);
        for (int i=0; i<in.length; i++) {
            String thisByte = Integer.toString(in[i], 16);
            if (thisByte.length() == 1)
                result.append('0');
            result.append(thisByte.toUpperCase());
        }
        return result.toString();
    }
    
    /**
     * Convert a hex string to a byte array
     * @param in
     * @return
     * @throws FormatException
     */
    public static int[] hexToByte(String in) throws FormatException{
        in = normalizeString(in);
        int len = in.length() / 2;
        int[] result = new int[len];
        if (in.length() % 2 != 0)
            throw new FormatException("The input hex string length is not even divisible by 2");
        for (int charPos=0; charPos < len; charPos++) {
            int value = 0;
            for (int bitPos = 0; bitPos < 2; bitPos++) {
                char c = Character.toUpperCase(in.charAt(charPos * 2 + bitPos));
                if ((c >= '0') && (c <= '9')) {
                    value = value << 4;
                    value = value + c - '0';
                }else if ((c >= 'A') && (c <= 'F')) {
                    value = value << 4;
                    value = value + c - 'A' + 10;
                }else{
                    throw new FormatException("Non-hex character found in data: " + c);
                }
            }
            result[charPos] = value;
        }
        return result;
    }
    
    /**
     * Convert a byte array to ASCII
     * @param in
     * @return
     */
    public static String byteToAscii(int[] in) {
        StringBuffer result = new StringBuffer(in.length);
        for (int i=0; i<in.length; i++) {
            result.append(String.valueOf((char) in[i]));
        }
        return result.toString();
    }
    
    /**
     * Convert an ASCII string to a byte array
     * @param in
     * @return
     */
    public static int[] asciiToByte(String in) {
        int[] result = new int[in.length()];
        for (int i=0; i<in.length(); i++)
            result[i] = in.charAt(i);
        return result;
    }
    
    /**
     * Convert a byte array to printable ASCII in HTML form (red question marks
     * for unprintable characters)
     * @param in
     * @return
     */
    public static String byteToPrintableAscii(int[] in) {
        StringBuffer result = new StringBuffer();
        result.append("<html><head></head><body><code>");
        for (int i=0; i<in.length; i++) {
            char c = (char) in[i];
            if ((c >= ' ') && (c <= '~'))
                result.append(c);
            else if (c == '\n')
                result.append("<br>");
            else if (c == '\r')
                ;
            else
                result.append("<font color=#ff0000>?</font>");
        }
        result.append("</code></body></html>");
        return result.toString();
    }
    
    /**
     * Normalize a string by removing all whitespace, control characters, and 
     * high-ASCII
     * @param in
     * @return
     */
    public static String normalizeString(String in) {
        StringBuffer result = new StringBuffer(in.length());
        for (int i=0; i<in.length(); i++) {
            char c = in.charAt(i);
            if ((c > ' ') && (c <= '~'))
                result.append(c);
        }
        return result.toString();
    }
}
